# $Id$
package Text::Template::YAWN;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

use Text::DelimMatch;
use IO::File;

@ISA = qw();
$VERSION = '0.01';

sub DEBUG () {0}

################################################################
sub new
{
    my($package, $start, $end)=@_;
    $start||='{';
    $end||='}';
    my $self={parser=>Text::DelimMatch->new($start, $end), parts=>{}};
    return bless $self, $package;
}

sub clear 
{
    my($self)=@_;
    $self->{parts}={};
}

################################################################
sub load
{
    my($self, $file)=@_;
    $self=$self->new() unless ref $self;
    $self->clear();
    return $self->add_file($file);
}

################################################################
sub add_file
{
    my($self, $file, $append)=@_;
    my $fh=$file;
    my $data=eval {                         ## Read in all the data
        unless(ref $fh) {
            $fh=IO::File->new($file);        
            if(not defined $fh) {
                die "Unable to open $file: $!\n";
            }
        }
        local $/;
        return <$fh>;        
    };
    return if $@;
    return $self->add_text('', $data, $append); ## and append it as-is
}


################################################################
sub split
{
    my($self, $text)=@_;
    my @q=$self->{parser}->match($text);    
    $q[0]=$text unless($q[1]);              # nothing matched

                                            # get all the matches up front so
    my @parts;                              # we can recurse with the parser
    my $last;
    while(grep defined, @q) {
        DEBUG and warn "0=", ($q[0]||''), " 1=", ($q[1]||''), " 2=", ($q[2]||'');
        push @parts, [@q[0..1]];
        $last=$q[2];
        @q=$self->{parser}->match($last);
        # warn "$last give ", join '-', @q;
    }
    push @parts, $last;
    return @parts;
}


################################################################
sub add_text
{
    my($self, $current, $text, $append)=@_;
    if(2==@_) {
        $text=$current;
        $current='';
        $append=0;
    }
    elsif($current) {
        $append||=0;
    } else {
        $append||=1;
    }

    if($current and (not $append or not $self->{parts}{$current})) {
        DEBUG and warn "Defining $current";
        $self->{parts}{$current}='';        
    }

    my($s, $e)=$self->{parser}->delim;
    $self->{parser}->delim($s, $e);         # *sigh*
    my @parts=$self->split($text);
    my $last=pop @parts if @parts;
    my($pre, $match);

    foreach my $q (@parts) {                # now process the parts
        ($pre, $match)=@$q;
#        $pre||='';
        DEBUG and warn "pre=$pre match=$match";
        if(defined $match) {                # is it a define block?
            if($match =~ m(^${s}define:(\w+)\s+(.+)$e$)s) {
                DEBUG and warn "define=$1, text=", substr($2, 0, 10);
                $self->add_text($1, $2, $append);
            } else {
                $pre.=$match;               # it could be a [text block]
            }
        }
        if($current) {
            $self->{parts}{$current}.=$pre;
        } elsif($pre=~/\S/) {
            $pre=substr($pre, 0, 20);
            warn "Text outside of a block near '$pre'\n";
        }
    }

    if($current and $last and $last =~ /\S/) {
        $self->{parts}{$current}.=$last;
    }
    return 1;
}

################################################################
sub set 
{
    my $self=shift;
    my($k, $v);
    while(@_) {
        $k=shift;
        $v=shift;
        $self->add_text($k, $v, 0);
    }
}
*add=\&set;

################################################################
sub render
{
    my($self, $current)=@_;
    my $text=$self->{parts}{$current};
    return $text unless $text;
    
    return $self->render_text($text, 0);
}
*get=\&render;

################################################################
sub render_text
{
    my($self, $text, $inside)=@_;
    unless(defined $text or $text ne '') {
        return if $inside;                  # potential voiding of sub-part
        return $text unless $inside;        # sink or swim
    }

    $inside=0 if @_==2;

    my @parts=$self->split($text);
    my $last=pop @parts if @parts;

    my($s, $e)=$self->{parser}->delim;
    $self->{parser}->delim($s, $e);         # *sigh*
    my($pre, $match, $w);
    $text='';

    foreach my $q (@parts) {                # now process the parts
        ($pre, $match)=@$q;

        $text.=$pre if defined $pre;        # building output

        next unless $match;
        $match =~ s/^$s//;                  # strip out delimiters
        $match =~ s/$e$//;

        if($match =~ /^(\w+)$/) {
            DEBUG and warn "Rendering '$1' from '$self->{parts}{$1}'";
            $w=$self->render_text($self->{parts}{$1}, 1)
                                            if defined $self->{parts}{$1};
        } else {
            $w=$self->render_text($match, 1);
        }
        DEBUG and warn "'$match' is '$w'";
        # here's the magic.  If we don't have this [sub_part],
        # it's container is "voided"
        return if $inside and not defined $w;
        $text.=$w if defined $w;
    }
    $text.=$last if $last;
    return $text;
}


################################################################
## Delegation 
sub _mk_delegate 
{
    my($func)=@_;
    my $sub;
    return sub {
        my $self=shift;
        $sub||=$self->{parser}->can($func);
        return $sub->($self->{parser}, @_);
    }
}

*delim=_mk_delegate('delim');
*quote=_mk_delegate('quote ');
*escape=_mk_delegate('escape');
*double_escape=_mk_delegate('double_escape ');
*case_sensitive=_mk_delegate('case_sensitive');
*keep =_mk_delegate('keep ');
*error=_mk_delegate('error');
*pre_matched=_mk_delegate('pre_matched');
*matched=_mk_delegate('matched');
*post_matched=_mk_delegate('post_matched');
*debug=_mk_delegate('debug');
*dump=_mk_delegate('dump');
*slow=_mk_delegate('slow');

################################################################

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Text::Template::YAWN - Text Templates : Yet Another Way Now

=head1 SYNOPSIS

    use Text::Template::YAWN;
    my $yawn=Text::Template::YAWN->new('<{', '}>');
    $yawn->load("data.yawn") or die $@;

    ... do some work that sets $hits, $start, $end ...
    
    $yawn->set(hits=>$hits, start=>$start, end=>$end);
    $yawn->set('s'=>'') if $hits==1;

    print $yawn->get('results);



=head1 DESCRIPTION

YAWN is Yet Another text template thingy.  Is this one better then the
others? No.  Is this one worse?  No.  Design goal for YAWN as small size and
(hopefully) speed.


=head1 AUTHOR

Philip Gwyn, perler@pied.nu

=head1 SEE ALSO

perl(1).

=cut
 
$Log$